
module generic_dpram(
    // Generic synchronous dual-port RAM interface
    rclk, rce, oe, raddr, do,
    wclk, wce, we, waddr, di
);

//
// Default address and data buses width
//
parameter aw = 5;  // number of bits in address-bus
parameter dw = 16; // number of bits in data-bus

//
// Generic synchronous double-port RAM interface
//
// read port
input           rclk;  // read clock, rising edge trigger
input           rce;   // read port chip enable, active high
input           oe;	   // output enable, active high
input  [aw-1:0] raddr; // read address
output [dw-1:0] do;    // data output

// write port
input          wclk;  // write clock, rising edge trigger
input          wce;   // write port chip enable, active high
input          we;    // write enable, active high
input [aw-1:0] waddr; // write address
input [dw-1:0] di;    // data input

//
// Module body
//
reg [dw-1:0] mem [(1<<aw) -1:0] /* synthesis syn_ramstyle="no_rw_check" */;
reg [aw-1:0] ra;                // register read address

// read operation
always @(posedge rclk)
    if (rce)
        ra <= raddr;

assign do = mem[ra];

// write operation
always @(posedge wclk)
    if (we && wce)
        mem[waddr] <= di;

endmodule


module JTAG_REG_0 (en, tck, update, shift, tdi, tdo, dout);
parameter LENGTH=11;
input en, tck, update, shift, tdi;
output tdo;
output [LENGTH-1:0] dout; 
reg [LENGTH-1:0] data_reg1;
reg [LENGTH-1:0] data_reg2;
reg en_d;
	always @(negedge tck)
	begin  
		if (en & shift | en_d) begin    
			data_reg1 <= {data_reg1[LENGTH-2:0], tdi};
		end
		en_d <= en & shift;
	end
	always @(negedge tck)
	begin
		if (update) begin
			data_reg2 <= data_reg1;
		end
	end
	assign dout = data_reg2;
	assign tdo = data_reg1[LENGTH-1];
endmodule


module JTAG_REG_1 (en, tck, update, shift, tdo, din, dout);
parameter LENGTH=11;
input en, tck, update, shift;
input [LENGTH-1:0] din;
output [LENGTH-1:0] dout; 
output tdo;
reg [LENGTH-1:0] data_reg1;
reg [LENGTH-1:0] data_reg2;
reg en_d;
	always @(negedge tck)
	begin
	if (en & shift | en_d) begin    
		data_reg1 <= {data_reg1[LENGTH-1:0], 1'b0};
	end else begin
		data_reg2 <= data_reg1;
		data_reg1 <= din;
	end    
	en_d <= en & shift;    
	end
	assign dout = data_reg2;
	assign tdo = data_reg1[LENGTH-1];
endmodule


module EDGE (ctrl, ref_clk, reset, cond_in, cond_out);
parameter WIDTH = 1;
input [1:0] ctrl;
input ref_clk, cond_in,reset;
output cond_out;
reg cond_out;
reg cond_in_delay;
wire ignore;
	assign ignore = ctrl[1];
	always @(posedge ref_clk)
	begin
		if (reset) begin
			cond_in_delay <= ctrl[0];
		end else begin
			cond_in_delay <= cond_in;
		end        
	end
	always @(posedge ref_clk)
	begin
		if (reset) begin
			cond_out <= 1'b0;
		end else if (ignore) begin
			cond_out <= 1'b1;
		end else begin
			cond_out <= (ctrl[0]^~cond_in)&(cond_in^cond_in_delay);
		end            
	end
endmodule


module ARITHM (ctrl, ref_clk, reset, cond_in, cond_out);
parameter WIDTH=1;
input [WIDTH+4-1:0] ctrl;
input [WIDTH-1:0] cond_in;
input ref_clk, reset;
output cond_out;
reg cond_out;
wire ignore, eq, lt, gt, ne,eq_flag, ge_flag, lt_flag;
wire [WIDTH:0] res;
wire [WIDTH-1:0] operand;
	assign ignore = ctrl[WIDTH+3];
	assign eq = ctrl[WIDTH+2];
	assign lt = ctrl[WIDTH+1];
	assign gt = ctrl[WIDTH];
	assign ne = ~(|ctrl[WIDTH+2:WIDTH]);   //add by ll 
	assign operand = ctrl[WIDTH-1:0];
	assign res = {1'b1,cond_in} - operand;
	assign eq_flag = ~(|res[WIDTH-1:0]);
	assign ge_flag = res[WIDTH];
	assign lt_flag = ~res[WIDTH];
	always @(posedge ref_clk)
	begin
		if (reset) begin
			cond_out <= 1'b0;
		end else if (ignore) begin
			cond_out <= 1'b1;
		end else if (eq) begin
			if (lt) begin
				cond_out <= eq_flag | lt_flag;      // LE
			end else if (gt) begin
				cond_out <= ge_flag;                // GE
			end else begin
				cond_out <= eq_flag;                // EQ
			end
		end else if (lt) begin
			cond_out <= lt_flag;                    // LT
		end else if (gt) begin
			cond_out <= (~eq_flag) & ge_flag;       // GT
		end else if(ne) begin
			cond_out <= lt_flag | ((~eq_flag) & ge_flag);  //NE
		end else begin
			cond_out <= cond_out;
		end    
	end
endmodule


module MEM_CTRL (reset, trigger, wclk, rclk, re, waddr, raddr, wen, status, offset, reset_raddr);
parameter ADDR_WIDTH = 9;
input reset, trigger, wclk, rclk, re;
input reset_raddr;
input [ADDR_WIDTH-1:0] offset;
output [ADDR_WIDTH-1:0] waddr;
output [ADDR_WIDTH-1:0] raddr;
output [ADDR_WIDTH+2-1:0] status;
output wen;
reg [ADDR_WIDTH-1:0] waddr;
reg [ADDR_WIDTH-1:0] raddr;
reg [ADDR_WIDTH-1:0] depth_count;
reg cout;
reg overflow;
reg done;
reg wen;
reg trigger_d;
reg capture;
	assign status[ADDR_WIDTH+1] = overflow;
	assign status[ADDR_WIDTH] = done;
	assign status[ADDR_WIDTH-1:0] = waddr;
	// wen generate;
	always @(posedge wclk)
	begin
		if (reset) begin
			wen <= 1'b1;
		end else if (done) begin
			wen <= 1'b0;
		end else begin
			wen <= wen;
		end
	end
	// waddr generator;
	always @(posedge wclk)
	begin
		if (reset) begin
			waddr <= 0;
			cout <= 1'b0;
		end else if (done == 1'b0) begin
			{cout, waddr} <= waddr + 1;
		end else begin
			waddr <= waddr;
		end           
	end    
	// overflow generate;        
	always @(posedge wclk)
	begin
		if (reset) begin
			overflow <= 1'b0;
		end else if (cout) begin
			overflow <= 1'b1;
		end else begin 
			overflow <= overflow;
		end            
	end
	// generate capture signal;
	always @(posedge wclk)
	begin
		if (reset) begin
			capture <= 1'b0;
		end else if (done==1'b0) begin
			if (trigger&(~trigger_d)) begin
				capture <= 1'b1;
			end else begin
			capture <= capture;
			end
		end else begin
			capture <= capture;
		end    
	end
	always @(posedge wclk)
	begin
		if (reset) begin
			trigger_d <= 1'b1;
		end else begin
			trigger_d <= trigger;
		end    
	end
	// depth counter;
	always @(posedge wclk)
	begin
		if (reset) begin
			depth_count <= offset;
			done <= 1'b0;
		end else if (capture) begin
			if (!done) begin
				{done, depth_count} <= {done, depth_count} + 1;
			end
		end else begin 
			done <= done;
			depth_count <= depth_count;
		end        
	end
	// raddr generator;
	reg reset_raddr_d;
	reg re_d;

	always @(posedge rclk)
	begin
		reset_raddr_d = reset_raddr;
		re_d = re;
	end

	wire reset_raddr_p;
	wire re_p;
	assign reset_raddr_p = reset_raddr & ~reset_raddr_d;
	assign re_p = re & ~re_d;

	always @(posedge rclk)
	begin
		if (reset_raddr_p) begin
			raddr <= 0;
		end else if (re_p) begin
			raddr <= raddr + 1;
		end    	
	end
endmodule


module RANGE (ctrl, ref_clk, reset, cond_in, cond_out);
parameter WIDTH=1;
//input [2*WIDTH+4+4-1:0] ctrl;
input [2*WIDTH+4+4:0] ctrl;
input [WIDTH-1:0] cond_in;
input ref_clk, reset;
output cond_out;
wire c1,c2;

wire [1:0] range_expr;
assign range_expr[0] = c1&c2;	//in range 
assign range_expr[1] = c1|c2;	//not in range 

assign cond_out = range_expr[ctrl[2*WIDTH+4+4]];
	ARITHM u_A1 (
		.reset(reset),
		.ctrl(ctrl[WIDTH+4-1:0]), 
		.ref_clk(ref_clk), 
		.cond_in(cond_in), 
		.cond_out(c1)
		);
	defparam u_A1.WIDTH = WIDTH;
	ARITHM u_A2 (
		.reset(reset),
		.ctrl(ctrl[2*WIDTH+4+4-1:WIDTH+4]), 
		.ref_clk(ref_clk), 
		.cond_in(cond_in), 
		.cond_out(c2)
		);
	defparam u_A2.WIDTH = WIDTH;
	 //assign cond_out = c1&c2;
endmodule

module TAP (tck1, tdi1, tdo1, sel1, tck2, tdi2, tdo2, sel2, 
            update, capture, shift, reset,
            TMS, TDI, TDO, TCK);

input TCK, TMS, TDI;
output TDO;
input tdo1, tdo2;
output tck1, tck2, tdi1, sel1, tdi2, sel2, shift, reset, update, capture;
wire tdi;
	wire jtagf_reset;
	JTAGF jtagf_inst ( 

		//output
		.TDO        (TDO),
		.JTCK				(tck2),
		.JTDI				(tdi),
		.JCE1				(sel1),
		.JCE2				(sel2),
		.JRTI1			(),
		.JRTI2			(),
		.JRSTN			(jtagf_reset),
		.JUPDATE		(update),
		.JSHIFT			(shift),

		//Input
		.TCK		    (TCK),
		.TMS				(TMS),
		.TDI				(TDI),
		.JTDO1			(tdo1),
		.JTDO2			(tdo2)
	); 
	assign reset = ~jtagf_reset;
	assign tdi1 = tdi;
	assign tdi2 = tdi;
	assign tck1 = tck2;
	assign capture = 1'b0;
	endmodule

module LA_MANAGER (tck1, tdi1, tdo1, sel1, tck2, tdi2, tdo2, sel2, update, shift, reset, la_sel, status, ctrl, mem, offset, la_reset, la_tck, la_update, la_shift, la_tdi, la_tdo);
parameter NUM_OF_LA = 2;
input tck1, tdi1, sel1, tck2, tdi2, sel2, update, shift, reset;
input [NUM_OF_LA-1:0] la_tdo;
output tdo1, tdo2, status, ctrl, mem, offset, la_reset, la_tck, la_update, la_shift, la_tdi;
output [NUM_OF_LA-1:0] la_sel;
wire [NUM_OF_LA+8-1:0] dout;
wire [2:0] opcode;
	JTAG_REG_0 u_cmd (
		.en(sel1),
		.tck(tck1),
		.update(update),
		.shift(shift),
		.tdi(tdi1),
		.tdo(tdo1),
		.dout(dout)
	);
	 defparam u_cmd.LENGTH= NUM_OF_LA+8;
	assign opcode = dout[NUM_OF_LA+7: NUM_OF_LA+5];
	assign offset = dout[NUM_OF_LA+3]&sel2;
	assign status = dout[NUM_OF_LA+2]&sel2;
	assign ctrl = dout[NUM_OF_LA+1]&sel2;
	assign mem = dout[NUM_OF_LA];
	assign la_reset = (&opcode);   // opcode = 3'b111;
	assign la_sel = dout[NUM_OF_LA-1:0];
	assign la_tck = tck2;
	assign la_update = update;
	assign la_shift = shift;
	assign la_tdi = tdi2;
	assign tdo2 = |(la_sel&la_tdo);
endmodule
module INSIGHT(trig_in_0, data_in_0, ref_clk_0,
            TCK, TMS, TDI, TDO
);

input TCK, TMS, TDI;
output TDO;
input [26:0] trig_in_0;
input [116:0] data_in_0;
input ref_clk_0;
wire tck1, tdi1, tdo1, sel1, tck2, tdi2, tdo2, sel2, update, shift, reset, capture;
wire [0:0] la_sel;
wire [0:0] la_tdo;
wire status, ctrl, offset, mem, la_reset, la_tck, la_tdi, la_shift, la_update;
	TAP u_tap(
		.TCK(TCK),
		.TDI(TDI),
		.TMS(TMS),
		.TDO(TDO),
		.tck1(tck1),
		.tdi1(tdi1),
		.tdo1(tdo1),
		.sel1(sel1),
		.tck2(tck2),
		.tdi2(tdi2),
		.tdo2(tdo2),
		.sel2(sel2),
		.update(update),
		.shift(shift),
		.reset(reset),
		.capture(capture)
	);
	LA_MANAGER u_la_manager(
		.tck1(tck1),
		.tdi1(tdi1),
		.tdo1(tdo1),
		.sel1(sel1),
		.tck2(tck2),
		.tdi2(tdi2),
		.tdo2(tdo2),
		.sel2(sel2),
		.update(update),
		.shift(shift),
		.reset(reset),
		.la_sel(la_sel),
		.status(status),
		.ctrl(ctrl),
		.mem(mem),
		.offset(offset),
		.la_reset(la_reset),
		.la_tck(la_tck),
		.la_update(la_update),
		.la_shift(la_shift),
		.la_tdi(la_tdi),
		.la_tdo(la_tdo)
	);
	defparam u_la_manager.NUM_OF_LA = 1;
	LA_CORE_0 u_la_core_0 (
		.sel(la_sel[0]),
		.status_sel(status),
		.ctrl_sel(ctrl),
		.mem_sel(mem&sel2),
		.offset_sel(offset),
		.reset(la_reset),
		.tck(la_tck),
		.update(la_update),
		.shift(la_shift),
		.tdi(la_tdi),
		.tdo(la_tdo[0]),
		.trig_in(trig_in_0),
		.data_in(data_in_0[116:0]),
		.ref_clk(ref_clk_0),
		.reset_raddr(mem&la_sel[0])
	);
endmodule
module LA_CORE_0 (ref_clk, trig_in, data_in, tck, tdi, tdo, update, shift, status_sel, ctrl_sel, mem_sel, offset_sel, sel, reset, reset_raddr);
input ref_clk, tck, tdi, update, shift, status_sel, ctrl_sel, mem_sel, offset_sel, sel, reset, reset_raddr;
input [26:0] trig_in;
input [116:0] data_in;
output tdo;
wire trig_out;
wire [199:0] ctrl;
wire [14:0] status;
wire [116:0] mem;
wire [12:0] offset;
wire ctrl_tdo, status_tdo, mem_tdo, offset_tdo;
reg  update_d;
	always @(negedge tck)
		update_d = update;
	assign tdo = (ctrl_tdo & ctrl_sel) | (status_tdo & status_sel) | (mem_tdo & mem_sel) | (offset_tdo & offset_sel);
	JTAG_REG_0 u_ctrl(
		.en(ctrl_sel&sel),
		.tck(tck),
		.update(update),
		.shift(shift),
		.tdi(tdi),
		.tdo(ctrl_tdo),
//		.din(),
		.dout(ctrl)
	);
	defparam u_ctrl.LENGTH=200;
	JTAG_REG_1 u_status(
		.en(status_sel&sel),
		.tck(tck),
		.update(update),
		.shift(shift),
//		.tdi(1'b0),
		.tdo(status_tdo),
		.din(status),
		.dout()
	);
	defparam u_status.LENGTH=15;
	JTAG_REG_1 u_mem(
		.en(mem_sel&sel),
		.tck(tck),
		.update(update),
		.shift(shift),
//		.tdi(tdi),
		.tdo(mem_tdo),
		.din(mem),
		.dout()
	);
	defparam u_mem.LENGTH=117;
	JTAG_REG_0 u_offset(
		.en(offset_sel&sel),
		.tck(tck),
		.update(update),
		.shift(shift),
		.tdi(tdi),
		.tdo(offset_tdo),
//		.din(),
		.dout(offset)
	);
	defparam u_offset.LENGTH=13;
	TRIGGER_0 u_trigger_0(
		.reset(~update&update_d&reset),
		.ctrl(ctrl),
		.ref_clk(ref_clk),
		.trig_in(trig_in),
		.trig_out(trig_out)
	);
	STORAGE_0 u_stroage_0(
		.reset(~update&update_d&reset),
		.ref_clk(ref_clk),
		.trigger(trig_out),
		.data_in(data_in),
		.mem_read(update_d & reset_raddr),
		.mem_clk(~tck),
		.mem_out(mem),
		.status(status),
		.offset(offset),
		.reset_raddr(reset_raddr)
	);
endmodule  // end of LA module
module TRIGGER_0(ref_clk, reset, trig_in, trig_out, ctrl);
input ref_clk;
input reset;
input [26:0]trig_in;
input [199:0]ctrl;
output trig_out;
wire [8:0]cond_out;
wire [3:0] expression;    //add by ll

	assign expression[0] = &cond_out;
	assign expression[1] = ~(&cond_out);
	assign expression[2] = |cond_out;
	assign expression[3] = ~(|cond_out);

	assign trig_out = expression[ctrl[199:198]];
	TRIGGER_UNIT_0  u_trig_unit_0 (
		.reset(reset),
		.ref_clk(ref_clk),
		.ctrl(ctrl[197:176]),
		.cond_in(trig_in[26:24]),
		.cond_out(cond_out[0])
	);
	TRIGGER_UNIT_1  u_trig_unit_1 (
		.reset(reset),
		.ref_clk(ref_clk),
		.ctrl(ctrl[175:154]),
		.cond_in(trig_in[23:21]),
		.cond_out(cond_out[1])
	);
	TRIGGER_UNIT_2  u_trig_unit_2 (
		.reset(reset),
		.ref_clk(ref_clk),
		.ctrl(ctrl[153:132]),
		.cond_in(trig_in[20:18]),
		.cond_out(cond_out[2])
	);
	TRIGGER_UNIT_3  u_trig_unit_3 (
		.reset(reset),
		.ref_clk(ref_clk),
		.ctrl(ctrl[131:110]),
		.cond_in(trig_in[17:15]),
		.cond_out(cond_out[3])
	);
	TRIGGER_UNIT_4  u_trig_unit_4 (
		.reset(reset),
		.ref_clk(ref_clk),
		.ctrl(ctrl[109:88]),
		.cond_in(trig_in[14:12]),
		.cond_out(cond_out[4])
	);
	TRIGGER_UNIT_5  u_trig_unit_5 (
		.reset(reset),
		.ref_clk(ref_clk),
		.ctrl(ctrl[87:66]),
		.cond_in(trig_in[11:9]),
		.cond_out(cond_out[5])
	);
	TRIGGER_UNIT_6  u_trig_unit_6 (
		.reset(reset),
		.ref_clk(ref_clk),
		.ctrl(ctrl[65:44]),
		.cond_in(trig_in[8:6]),
		.cond_out(cond_out[6])
	);
	TRIGGER_UNIT_7  u_trig_unit_7 (
		.reset(reset),
		.ref_clk(ref_clk),
		.ctrl(ctrl[43:22]),
		.cond_in(trig_in[5:3]),
		.cond_out(cond_out[7])
	);
	TRIGGER_UNIT_8  u_trig_unit_8 (
		.reset(reset),
		.ref_clk(ref_clk),
		.ctrl(ctrl[21:0]),
		.cond_in(trig_in[2:0]),
		.cond_out(cond_out[8])
	);
endmodule
module TRIGGER_UNIT_0(reset, ref_clk, ctrl, cond_in, cond_out);
input reset;
input ref_clk;
input [21:0]ctrl;
input [2:0]cond_in;
output cond_out;
wire [2:0]trig_cond_out;
wire [15:0] expression;    //add by ll
	assign expression[0] = 1;//add by ll
	assign expression[1] = 1;
	assign expression[2] = 0;
	assign expression[3] = 0;
	//dummy
	assign expression[4] = 0;
	assign expression[5] = 0;
	assign expression[6] = 0;
	assign expression[7] = 0;
	assign expression[8] = 0;
	assign expression[9] = 0;
	//dummy
	assign expression[10] = &trig_cond_out;
	assign expression[11] = &trig_cond_out;
	assign expression[12] = &trig_cond_out;
	assign expression[13] = &trig_cond_out;
	//dummy
	assign expression[14] = 0;
	assign expression[15] = 0;

	assign cond_out = expression[ctrl[20:17]];
	EDGE u_cond_0 (
		.reset(reset),
		.ref_clk(ref_clk),
		.ctrl(ctrl[16:15]),
		.cond_in(cond_in[2]),
		.cond_out(trig_cond_out[0])
	);
	defparam u_cond_0.WIDTH=1;
	RANGE u_cond_1 (
		.reset(reset),
		.ref_clk(ref_clk),
		.ctrl({ctrl[21],ctrl[14:5]}),
		.cond_in(cond_in[1:1]),
		.cond_out(trig_cond_out[1])
	);
	defparam u_cond_1.WIDTH=1;
	ARITHM u_cond_2 (
		.reset(reset),
		.ref_clk(ref_clk),
		.ctrl(ctrl[4:0]),
		.cond_in(cond_in[0:0]),
		.cond_out(trig_cond_out[2])
	);
	defparam u_cond_2.WIDTH=1;
endmodule
module TRIGGER_UNIT_1(reset, ref_clk, ctrl, cond_in, cond_out);
input reset;
input ref_clk;
input [21:0]ctrl;
input [2:0]cond_in;
output cond_out;
wire [2:0]trig_cond_out;
wire [15:0] expression;    //add by ll
	assign expression[0] = 1;//add by ll
	assign expression[1] = 1;
	assign expression[2] = 0;
	assign expression[3] = 0;
	//dummy
	assign expression[4] = 0;
	assign expression[5] = 0;
	assign expression[6] = 0;
	assign expression[7] = 0;
	assign expression[8] = 0;
	assign expression[9] = 0;
	//dummy
	assign expression[10] = &trig_cond_out;
	assign expression[11] = &trig_cond_out;
	assign expression[12] = &trig_cond_out;
	assign expression[13] = &trig_cond_out;
	//dummy
	assign expression[14] = 0;
	assign expression[15] = 0;

	assign cond_out = expression[ctrl[20:17]];
	EDGE u_cond_0 (
		.reset(reset),
		.ref_clk(ref_clk),
		.ctrl(ctrl[16:15]),
		.cond_in(cond_in[2]),
		.cond_out(trig_cond_out[0])
	);
	defparam u_cond_0.WIDTH=1;
	RANGE u_cond_1 (
		.reset(reset),
		.ref_clk(ref_clk),
		.ctrl({ctrl[21],ctrl[14:5]}),
		.cond_in(cond_in[1:1]),
		.cond_out(trig_cond_out[1])
	);
	defparam u_cond_1.WIDTH=1;
	ARITHM u_cond_2 (
		.reset(reset),
		.ref_clk(ref_clk),
		.ctrl(ctrl[4:0]),
		.cond_in(cond_in[0:0]),
		.cond_out(trig_cond_out[2])
	);
	defparam u_cond_2.WIDTH=1;
endmodule
module TRIGGER_UNIT_2(reset, ref_clk, ctrl, cond_in, cond_out);
input reset;
input ref_clk;
input [21:0]ctrl;
input [2:0]cond_in;
output cond_out;
wire [2:0]trig_cond_out;
wire [15:0] expression;    //add by ll
	assign expression[0] = 1;//add by ll
	assign expression[1] = 1;
	assign expression[2] = 0;
	assign expression[3] = 0;
	//dummy
	assign expression[4] = 0;
	assign expression[5] = 0;
	assign expression[6] = 0;
	assign expression[7] = 0;
	assign expression[8] = 0;
	assign expression[9] = 0;
	//dummy
	assign expression[10] = &trig_cond_out;
	assign expression[11] = &trig_cond_out;
	assign expression[12] = &trig_cond_out;
	assign expression[13] = &trig_cond_out;
	//dummy
	assign expression[14] = 0;
	assign expression[15] = 0;

	assign cond_out = expression[ctrl[20:17]];
	EDGE u_cond_0 (
		.reset(reset),
		.ref_clk(ref_clk),
		.ctrl(ctrl[16:15]),
		.cond_in(cond_in[2]),
		.cond_out(trig_cond_out[0])
	);
	defparam u_cond_0.WIDTH=1;
	RANGE u_cond_1 (
		.reset(reset),
		.ref_clk(ref_clk),
		.ctrl({ctrl[21],ctrl[14:5]}),
		.cond_in(cond_in[1:1]),
		.cond_out(trig_cond_out[1])
	);
	defparam u_cond_1.WIDTH=1;
	ARITHM u_cond_2 (
		.reset(reset),
		.ref_clk(ref_clk),
		.ctrl(ctrl[4:0]),
		.cond_in(cond_in[0:0]),
		.cond_out(trig_cond_out[2])
	);
	defparam u_cond_2.WIDTH=1;
endmodule
module TRIGGER_UNIT_3(reset, ref_clk, ctrl, cond_in, cond_out);
input reset;
input ref_clk;
input [21:0]ctrl;
input [2:0]cond_in;
output cond_out;
wire [2:0]trig_cond_out;
wire [15:0] expression;    //add by ll
	assign expression[0] = 1;//add by ll
	assign expression[1] = 1;
	assign expression[2] = 0;
	assign expression[3] = 0;
	//dummy
	assign expression[4] = 0;
	assign expression[5] = 0;
	assign expression[6] = 0;
	assign expression[7] = 0;
	assign expression[8] = 0;
	assign expression[9] = 0;
	//dummy
	assign expression[10] = &trig_cond_out;
	assign expression[11] = &trig_cond_out;
	assign expression[12] = &trig_cond_out;
	assign expression[13] = &trig_cond_out;
	//dummy
	assign expression[14] = 0;
	assign expression[15] = 0;

	assign cond_out = expression[ctrl[20:17]];
	EDGE u_cond_0 (
		.reset(reset),
		.ref_clk(ref_clk),
		.ctrl(ctrl[16:15]),
		.cond_in(cond_in[2]),
		.cond_out(trig_cond_out[0])
	);
	defparam u_cond_0.WIDTH=1;
	RANGE u_cond_1 (
		.reset(reset),
		.ref_clk(ref_clk),
		.ctrl({ctrl[21],ctrl[14:5]}),
		.cond_in(cond_in[1:1]),
		.cond_out(trig_cond_out[1])
	);
	defparam u_cond_1.WIDTH=1;
	ARITHM u_cond_2 (
		.reset(reset),
		.ref_clk(ref_clk),
		.ctrl(ctrl[4:0]),
		.cond_in(cond_in[0:0]),
		.cond_out(trig_cond_out[2])
	);
	defparam u_cond_2.WIDTH=1;
endmodule
module TRIGGER_UNIT_4(reset, ref_clk, ctrl, cond_in, cond_out);
input reset;
input ref_clk;
input [21:0]ctrl;
input [2:0]cond_in;
output cond_out;
wire [2:0]trig_cond_out;
wire [15:0] expression;    //add by ll
	assign expression[0] = 1;//add by ll
	assign expression[1] = 1;
	assign expression[2] = 0;
	assign expression[3] = 0;
	//dummy
	assign expression[4] = 0;
	assign expression[5] = 0;
	assign expression[6] = 0;
	assign expression[7] = 0;
	assign expression[8] = 0;
	assign expression[9] = 0;
	//dummy
	assign expression[10] = &trig_cond_out;
	assign expression[11] = &trig_cond_out;
	assign expression[12] = &trig_cond_out;
	assign expression[13] = &trig_cond_out;
	//dummy
	assign expression[14] = 0;
	assign expression[15] = 0;

	assign cond_out = expression[ctrl[20:17]];
	EDGE u_cond_0 (
		.reset(reset),
		.ref_clk(ref_clk),
		.ctrl(ctrl[16:15]),
		.cond_in(cond_in[2]),
		.cond_out(trig_cond_out[0])
	);
	defparam u_cond_0.WIDTH=1;
	RANGE u_cond_1 (
		.reset(reset),
		.ref_clk(ref_clk),
		.ctrl({ctrl[21],ctrl[14:5]}),
		.cond_in(cond_in[1:1]),
		.cond_out(trig_cond_out[1])
	);
	defparam u_cond_1.WIDTH=1;
	ARITHM u_cond_2 (
		.reset(reset),
		.ref_clk(ref_clk),
		.ctrl(ctrl[4:0]),
		.cond_in(cond_in[0:0]),
		.cond_out(trig_cond_out[2])
	);
	defparam u_cond_2.WIDTH=1;
endmodule
module TRIGGER_UNIT_5(reset, ref_clk, ctrl, cond_in, cond_out);
input reset;
input ref_clk;
input [21:0]ctrl;
input [2:0]cond_in;
output cond_out;
wire [2:0]trig_cond_out;
wire [15:0] expression;    //add by ll
	assign expression[0] = 1;//add by ll
	assign expression[1] = 1;
	assign expression[2] = 0;
	assign expression[3] = 0;
	//dummy
	assign expression[4] = 0;
	assign expression[5] = 0;
	assign expression[6] = 0;
	assign expression[7] = 0;
	assign expression[8] = 0;
	assign expression[9] = 0;
	//dummy
	assign expression[10] = &trig_cond_out;
	assign expression[11] = &trig_cond_out;
	assign expression[12] = &trig_cond_out;
	assign expression[13] = &trig_cond_out;
	//dummy
	assign expression[14] = 0;
	assign expression[15] = 0;

	assign cond_out = expression[ctrl[20:17]];
	EDGE u_cond_0 (
		.reset(reset),
		.ref_clk(ref_clk),
		.ctrl(ctrl[16:15]),
		.cond_in(cond_in[2]),
		.cond_out(trig_cond_out[0])
	);
	defparam u_cond_0.WIDTH=1;
	RANGE u_cond_1 (
		.reset(reset),
		.ref_clk(ref_clk),
		.ctrl({ctrl[21],ctrl[14:5]}),
		.cond_in(cond_in[1:1]),
		.cond_out(trig_cond_out[1])
	);
	defparam u_cond_1.WIDTH=1;
	ARITHM u_cond_2 (
		.reset(reset),
		.ref_clk(ref_clk),
		.ctrl(ctrl[4:0]),
		.cond_in(cond_in[0:0]),
		.cond_out(trig_cond_out[2])
	);
	defparam u_cond_2.WIDTH=1;
endmodule
module TRIGGER_UNIT_6(reset, ref_clk, ctrl, cond_in, cond_out);
input reset;
input ref_clk;
input [21:0]ctrl;
input [2:0]cond_in;
output cond_out;
wire [2:0]trig_cond_out;
wire [15:0] expression;    //add by ll
	assign expression[0] = 1;//add by ll
	assign expression[1] = 1;
	assign expression[2] = 0;
	assign expression[3] = 0;
	//dummy
	assign expression[4] = 0;
	assign expression[5] = 0;
	assign expression[6] = 0;
	assign expression[7] = 0;
	assign expression[8] = 0;
	assign expression[9] = 0;
	//dummy
	assign expression[10] = &trig_cond_out;
	assign expression[11] = &trig_cond_out;
	assign expression[12] = &trig_cond_out;
	assign expression[13] = &trig_cond_out;
	//dummy
	assign expression[14] = 0;
	assign expression[15] = 0;

	assign cond_out = expression[ctrl[20:17]];
	EDGE u_cond_0 (
		.reset(reset),
		.ref_clk(ref_clk),
		.ctrl(ctrl[16:15]),
		.cond_in(cond_in[2]),
		.cond_out(trig_cond_out[0])
	);
	defparam u_cond_0.WIDTH=1;
	RANGE u_cond_1 (
		.reset(reset),
		.ref_clk(ref_clk),
		.ctrl({ctrl[21],ctrl[14:5]}),
		.cond_in(cond_in[1:1]),
		.cond_out(trig_cond_out[1])
	);
	defparam u_cond_1.WIDTH=1;
	ARITHM u_cond_2 (
		.reset(reset),
		.ref_clk(ref_clk),
		.ctrl(ctrl[4:0]),
		.cond_in(cond_in[0:0]),
		.cond_out(trig_cond_out[2])
	);
	defparam u_cond_2.WIDTH=1;
endmodule
module TRIGGER_UNIT_7(reset, ref_clk, ctrl, cond_in, cond_out);
input reset;
input ref_clk;
input [21:0]ctrl;
input [2:0]cond_in;
output cond_out;
wire [2:0]trig_cond_out;
wire [15:0] expression;    //add by ll
	assign expression[0] = 1;//add by ll
	assign expression[1] = 1;
	assign expression[2] = 0;
	assign expression[3] = 0;
	//dummy
	assign expression[4] = 0;
	assign expression[5] = 0;
	assign expression[6] = 0;
	assign expression[7] = 0;
	assign expression[8] = 0;
	assign expression[9] = 0;
	//dummy
	assign expression[10] = &trig_cond_out;
	assign expression[11] = &trig_cond_out;
	assign expression[12] = &trig_cond_out;
	assign expression[13] = &trig_cond_out;
	//dummy
	assign expression[14] = 0;
	assign expression[15] = 0;

	assign cond_out = expression[ctrl[20:17]];
	EDGE u_cond_0 (
		.reset(reset),
		.ref_clk(ref_clk),
		.ctrl(ctrl[16:15]),
		.cond_in(cond_in[2]),
		.cond_out(trig_cond_out[0])
	);
	defparam u_cond_0.WIDTH=1;
	RANGE u_cond_1 (
		.reset(reset),
		.ref_clk(ref_clk),
		.ctrl({ctrl[21],ctrl[14:5]}),
		.cond_in(cond_in[1:1]),
		.cond_out(trig_cond_out[1])
	);
	defparam u_cond_1.WIDTH=1;
	ARITHM u_cond_2 (
		.reset(reset),
		.ref_clk(ref_clk),
		.ctrl(ctrl[4:0]),
		.cond_in(cond_in[0:0]),
		.cond_out(trig_cond_out[2])
	);
	defparam u_cond_2.WIDTH=1;
endmodule
module TRIGGER_UNIT_8(reset, ref_clk, ctrl, cond_in, cond_out);
input reset;
input ref_clk;
input [21:0]ctrl;
input [2:0]cond_in;
output cond_out;
wire [2:0]trig_cond_out;
wire [15:0] expression;    //add by ll
	assign expression[0] = 1;//add by ll
	assign expression[1] = 1;
	assign expression[2] = 0;
	assign expression[3] = 0;
	//dummy
	assign expression[4] = 0;
	assign expression[5] = 0;
	assign expression[6] = 0;
	assign expression[7] = 0;
	assign expression[8] = 0;
	assign expression[9] = 0;
	//dummy
	assign expression[10] = &trig_cond_out;
	assign expression[11] = &trig_cond_out;
	assign expression[12] = &trig_cond_out;
	assign expression[13] = &trig_cond_out;
	//dummy
	assign expression[14] = 0;
	assign expression[15] = 0;

	assign cond_out = expression[ctrl[20:17]];
	EDGE u_cond_0 (
		.reset(reset),
		.ref_clk(ref_clk),
		.ctrl(ctrl[16:15]),
		.cond_in(cond_in[2]),
		.cond_out(trig_cond_out[0])
	);
	defparam u_cond_0.WIDTH=1;
	RANGE u_cond_1 (
		.reset(reset),
		.ref_clk(ref_clk),
		.ctrl({ctrl[21],ctrl[14:5]}),
		.cond_in(cond_in[1:1]),
		.cond_out(trig_cond_out[1])
	);
	defparam u_cond_1.WIDTH=1;
	ARITHM u_cond_2 (
		.reset(reset),
		.ref_clk(ref_clk),
		.ctrl(ctrl[4:0]),
		.cond_in(cond_in[0:0]),
		.cond_out(trig_cond_out[2])
	);
	defparam u_cond_2.WIDTH=1;
endmodule
module STORAGE_0(reset, ref_clk, trigger, data_in, mem_read, mem_clk, mem_out, status, offset, reset_raddr);
input reset, ref_clk, trigger, mem_clk, mem_read, reset_raddr;
input [116:0] data_in;
input [12:0] offset;
output [14:0] status;
output [116:0] mem_out;
wire [12:0] waddr;
wire [12:0] raddr;
wire wen;
	MEM_CTRL u_mem_ctrl_0(
		.reset(reset),
		.trigger(trigger),
		.wclk(ref_clk),
		.rclk(mem_clk),
		.re(mem_read),
		.waddr(waddr),
		.raddr(raddr),
		.wen(wen),
		.status(status),
		.offset(offset),
		.reset_raddr(reset_raddr)
	);
	defparam u_mem_ctrl_0.ADDR_WIDTH=13;
	MEMORY_0 u_memory_0 (
		.clka(ref_clk),
		.wena(wen),
		.addra(waddr),
		.dina(data_in),
		.clkb(mem_clk),
		.wenb(1'b0),
		.addrb(raddr),
		.dinb(117'b0),
		.doutb(mem_out)
	);
	defparam u_memory_0.ADDR_WIDTH=13;
	defparam u_memory_0.DATA_WIDTH=117;
endmodule
module MEMORY_0 (doutb, addra, addrb, clka, clkb, dina, dinb, wena, wenb);
parameter ADDR_WIDTH = 9;
parameter DATA_WIDTH = 8;
input [ADDR_WIDTH-1:0] addra,addrb;
input [DATA_WIDTH-1:0] dina,dinb;
output [DATA_WIDTH-1:0] doutb;
input clka,clkb,wena,wenb;
	generic_dpram  #(ADDR_WIDTH, DATA_WIDTH) mydpram0(		.rclk      (clkb),
		.rce       (1'b1),
		.oe        (1'b1),
		.raddr     (addrb),
		.do        (doutb),
		.wclk      (clka),
		.wce       (1'b1),
		.we        (wena),
		.waddr     (addra),
		.di        (dina));

endmodule
